#ifndef MYSPARSEMATRIX_H
#define MYSPARSEMATRIX_H

#include <cassert>
#include <vector>
#include <map>
#include <string>
#include <iostream>
#include <fstream>



#include <Eigen/SparseCore>





template< class float_t >
class mySparseMatrix{
    unsigned int nColumns , nRows;
    std::vector< std::map< unsigned int , float_t > > columns;
public:
    mySparseMatrix(  ) {
        nColumns = 0;
        nRows = 0;
        columns.clear();
        columns.resize(nColumns);
    }
    mySparseMatrix( unsigned int n_rows , unsigned int n_columns )  {
        nColumns = n_columns;
        nRows = n_rows;
        columns.clear();
        columns.resize(nColumns);
    }
    float_t & operator() (unsigned int row , unsigned int col) { return columns[col][row]; }
    float_t operator() (unsigned int row , unsigned int col) const  {
        typename std::map< unsigned int , float_t >::const_iterator it = columns[col].find(row);
        if( it == columns[col].end() ) return 0.0;
        return it->second;
    }

    template< class float_t2 >
    Eigen::SparseMatrix< float_t2 > toEigenSparse() const {
        Eigen::SparseMatrix< float_t2 > res( nRows , nColumns );
        std::vector< Eigen::Triplet< float_t2 > > triplets;
        for( unsigned int c = 0 ; c < nColumns ; ++c ) {
            for( typename std::map< unsigned int , float_t >::const_iterator it = columns[c].begin() ; it != columns[c].end() ; ++it ) {
                unsigned int r = it->first;
                float_t2 val = it->second;
                triplets.push_back( Eigen::Triplet< float_t2 >(r,c,val) );
            }
        }
        res.setFromTriplets( triplets.begin() , triplets.end() );
        return res;
    }
};




#endif // MYSPARSEMATRIX_H

